import unittest
import sys
import os
sys.path.insert(0, os.path.abspath("pyrg"))
import pyrg


class ColorFunctionTest(unittest.TestCase):

    def test_coloring_method(self):
        line = "get_gg (__main__.TestTest)"
        self.assertEqual("[36mget_gg (__main__.TestTest)[0m",
                         pyrg.coloring_method(line))

    def test_ngroute(self):
        pass
        input_strings = """..
----------------------------------------------------------------------
Ran 2 tests in 0.000s

OK
"""
        result_strings = """[32m.[0m[32m.[0m
----------------------------------------------------------------------
Ran 2 tests in 0.000s

[32mOK[0m"""
        ret = pyrg.parse_unittest_result(input_strings.splitlines(1))
        self.assertEqual(ret, result_strings)


class TestColor(unittest.TestCase):

    def setUp(self):
        self.test_color_define = ['black', 'gray', 'red', 'pink', 'darkred',
                             'green', 'yellowgreen', 'darkgreen', 'brown',
                             'yellow', 'gold', 'blue', 'lightblue', 'darkblue',
                             'magenta', 'lightmagenta', 'darkmagenta',
                             'cyan', 'lightcyan', 'darkcyan', 'silver',
                             'white', 'darksilver']
        #self.test_color_define_offset += 1
        None

    def test_colormap_key_black(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_gray(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_red(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_pink(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darkred(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_green(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_yellowgreen(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darkgreen(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_brown(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_yellow(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_gold(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_blue(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_lightblue(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darkblue(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_magenta(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_lightmagenta(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darkmagenta(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_cyan(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_lightcyan(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darkcyan(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_silver(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_white(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darksilver(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)


class TestDummy(unittest.TestCase):

    def test_dummy_fail(self):
        self.assertEqual(1, 2)

    def test_dummy_error(self):
        self.assertEqual(1, a)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""pyrg - colorized Python's UnitTest Result Tool"""
from ConfigParser import ConfigParser
from subprocess import Popen, PIPE
from select import poll, POLLIN
from optparse import OptionParser
import sys
import re
import os
import pwd

__version__ = '0.2.4'
__author__ = 'Hideo Hattroi <hhatto.jp@gmail.com>'
__license__ = 'NewBSDLicense'

__all__ = ['get_color', 'parse_result_line', 'parse_lineone',
           'coloring_method', 'parse_unittest_result', 'set_configuration']

DEFAULT_CONFIG_PATH = "/home/%s/.pyrgrc" % (pwd.getpwuid(os.getuid())[0])
PRINT_COLOR_SET_DEFAULT = {
        'ok': 'green',
        'fail': 'red',
        'error': 'yellow',
        'function': 'cyan',
}
PRINT_COLOR_SET = PRINT_COLOR_SET_DEFAULT
COLOR_MAP = {
        'black': '[30m%s[0m',
        'gray': '[1;30m%s[0m',
        #'black ': '[2;30m%s[0m',   ## not work
        'red': '[31m%s[0m',
        'pink': '[1;31m%s[0m',
        'darkred': '[2;31m%s[0m',
        'green': '[32m%s[0m',
        'yellowgreen': '[1;32m%s[0m',
        'darkgreen': '[2;32m%s[0m',
        'brown': '[33m%s[0m',
        'yellow': '[1;33m%s[0m',
        'gold': '[2;33m%s[0m',
        'blue': '[34m%s[0m',
        'lightblue': '[1;34m%s[0m',
        'darkblue': '[2;34m%s[0m',
        'magenta': '[35m%s[0m',
        'lightmagenta': '[1;35m%s[0m',
        'darkmagenta': '[2;35m%s[0m',
        'cyan': '[36m%s[0m',
        'lightcyan': '[1;36m%s[0m',
        'darkcyan': '[2;36m%s[0m',
        'silver': '[37m%s[0m',
        'white': '[1;37m%s[0m',
        'darksilver': '[2;37m%s[0m',
        }


def get_color(key):
    """color name get from COLOR_MAP dict."""
    return COLOR_MAP[PRINT_COLOR_SET[key]]


def parse_result_line(line):
    """parse to test result when fail tests"""
    err = False
    fail = False
    if 'errors' in line:
        err = True
    if 'failures' in line:
        fail = True
    if err and fail:
        f = line.split('=')[1].split(',')[0]
        e = line.split('=')[2].split(')')[0]
        result = "(%s=%s, " % (get_color('fail') % "failures",
                               get_color('fail') % f)
        result += "%s=%s)" % (get_color('error') % "errors",
                              get_color('error') % e)
    elif fail and not err:
        l = line.split('=')[1].split(')')[0]
        result = "(%s=%s)" % (get_color('fail') % "failures",
                              get_color('fail') % l)
    elif err and not fail:
        l = line.split('=')[1].split(')')[0]
        result = "(%s=%s)" % (get_color('error') % "errors",
                              get_color('error') % l)
    return get_color('fail') % "FAILED" + " %s" % result


def parse_lineone(line):
    """parse to test result line1"""
    results = []
    line = line.strip()
    for char in line:
        if '.' == char:
            results.append(get_color('ok') % ".")
        elif 'E' == char:
            results.append(get_color('error') % "E")
        elif 'F' == char:
            results.append(get_color('fail') % "F")
        else:
            results.append(char)
    return "".join(results)


def coloring_method(line):
    """colorized method line"""
    return get_color('function') % line


def parse_unittest_result(lines):
    """parse test result"""
    results = []
    err_verbose = re.compile("ERROR:")
    fail_verbose = re.compile("FAIL:")
    unittests_ok = re.compile("OK")
    unittests_failed = re.compile("FAILED")
    if not lines:
        return ""
    results.append(parse_lineone(lines[0]) + '\n')
    for line in lines[1:]:
        if unittests_ok.match(line):
            result = get_color('ok') % "OK"
        elif unittests_failed.match(line):
            result = parse_result_line(line)
        elif fail_verbose.match(line):
            result = "%s: %s\n" % (get_color('fail') % "FAIL",
                                   coloring_method(line[6:-1]))
        elif err_verbose.match(line):
            result = "%s: %s\n" % (get_color('error') % "ERROR",
                                   coloring_method(line[7:-1]))
        else:
            result = line
        results.append(result)
    return "".join(results)


def parse_unittest_result_verbose(lines):
    """parse test result, verbose print mode."""
    ok = re.compile("ok$")
    fail = re.compile("FAIL$")
    err = re.compile("ERROR$")
    fail_verbose = re.compile("FAIL:")
    err_verbose = re.compile("ERROR:")
    unittests_ok = re.compile("OK")
    unittests_failed = re.compile("FAILED")
    results = []
    for line in lines:
        if ok.search(line):
            tmp = ok.split(line)
            result = tmp[0] + get_color('ok') % "ok" + "\n"
        elif fail.search(line):
            tmp = fail.split(line)
            result = tmp[0] + get_color('fail') % "FAIL" + "\n"
        elif err.search(line):
            tmp = err.split(line)
            result = tmp[0] + get_color('error') % "ERROR" + "\n"
        elif fail_verbose.match(line):
            result = "%s: %s\n" % (get_color('fail') % "FAIL",
                                   coloring_method(line[6:-1]))
        elif err_verbose.match(line):
            result = "%s: %s\n" % (get_color('error') % "ERROR",
                                   coloring_method(line[7:-1]))
        elif unittests_ok.match(line):
            result = get_color('ok') % "OK"
        elif unittests_failed.match(line):
            result = parse_result_line(line)
        else:
            result = line
        results.append(result)
    return "".join(results)


def set_configuration(filename):
    """setting to printing color map"""
    ret = {}
    if not os.path.exists(filename):
        return PRINT_COLOR_SET_DEFAULT
    configure = ConfigParser()
    configure.read(filename)
    for item in configure.items('color'):
        if not item[0] in PRINT_COLOR_SET:
            continue
        if item[1] in COLOR_MAP:
            ret[item[0]] = item[1]
        else:
            ret[item[0]] = PRINT_COLOR_SET_DEFAULT[item[0]]
    return ret


def get_optionparser():
    """return to optparse's OptionParser object."""
    parser = OptionParser(version="pyrg: %s" % __version__,
                          description=__doc__,
                          usage="Usage: pyrg [options] TEST_SCRIPT.py\n"\
                                "     : python TEST_SCRIPT.py |& pyrg")
    parser.add_option('-v', '--verbose', action='store_true',
                      dest='mode_verbose',
                      help='print to verbose result for unittest.')
    parser.add_option('-d', '--default-color', action='store_true',
                      dest='mode_defaultcolor',
                      help='used to default color setting.')
    parser.add_option('-f', '--config-file', dest='config_filename',
                      help='configuration file path')
    return parser


def check_verbose(line):
    verbose = re.compile("(ok$|ERROR$|FAIL$)")
    return verbose.search(line)


def main():
    """execute command line tool"""
    global PRINT_COLOR_SET
    parser = get_optionparser()
    (opts, args) = parser.parse_args()
    if not opts.mode_defaultcolor:
        if opts.config_filename:
            PRINT_COLOR_SET = set_configuration(opts.config_filename)
        else:
            PRINT_COLOR_SET = set_configuration(DEFAULT_CONFIG_PATH)
    if len(args):
        if opts.mode_verbose:
            proc = Popen(['python', args[0], '-v'], stdout=PIPE, stderr=PIPE)
            result = proc.communicate()[1]
            print parse_unittest_result_verbose(result.splitlines(1))
        else:
            proc = Popen(['python', args[0]], stdout=PIPE, stderr=PIPE)
            result = proc.communicate()[1]
            print parse_unittest_result(result.splitlines(1))
    else:
        poller = poll()
        poller.register(sys.stdin, POLLIN)
        pollret = poller.poll(1)
        if len(pollret) == 1 and pollret[0][1] & POLLIN:
            lines = sys.stdin.readlines()
            if check_verbose(lines[0]):
                print parse_unittest_result_verbose(lines)
            else:
                print parse_unittest_result(lines)
        else:
            print parser.print_help()

if __name__ == '__main__':
    sys.exit(main())



import unittest
import sys
import os
from tempfile import NamedTemporaryFile
sys.path.insert(0, os.path.abspath("pyrg"))
import pyrg


class ColorFunctionTest(unittest.TestCase):

    def test_coloring_method(self):
        line = "get_gg (__main__.TestTest)"
        self.assertEqual("[36mget_gg (__main__.TestTest)[0m",
                         pyrg.coloring_method(line))

    def test_okroute(self):
        input_strings = """..
----------------------------------------------------------------------
Ran 2 tests in 0.000s

OK
"""
        result_strings = """[32m.[0m[32m.[0m
----------------------------------------------------------------------
Ran 2 tests in 0.000s

[32mOK[0m"""
        ret = pyrg.parse_unittest_result(input_strings.splitlines(1))
        self.assertEqual(ret, result_strings)

    def test_okroute_verbose(self):
        input_strings = """test_dummy1 (__main__.TestDummy) ... ok

----------------------------------------------------------------------
Ran 1 tests in 0.000s

OK
"""
        result_strings = """test_dummy1 (__main__.TestDummy) ... [32mok[0m

----------------------------------------------------------------------
Ran 1 tests in 0.000s

[32mOK[0m"""
        ret = pyrg.parse_unittest_result_verbose(input_strings.splitlines(1))
        self.assertEqual(ret, result_strings)

    def test_failroute(self):
        input_strings = """.F
======================================================================
FAIL: test_dummy_fail (__main__.TestDummy)
----------------------------------------------------------------------
Traceback (most recent call last):
  File "test/test_pyrg_ng.py", line 140, in test_dummy_fail
    self.assertEqual(1, 2)
AssertionError: 1 != 2

----------------------------------------------------------------------
Ran 2 tests in 0.000s

FAILED (failures=1)
"""
        result_strings = """[32m.[0m[31mF[0m
======================================================================
[31mFAIL[0m: [36mtest_dummy_fail (__main__.TestDummy)[0m
----------------------------------------------------------------------
Traceback (most recent call last):
  File "test/test_pyrg_ng.py", line 140, in test_dummy_fail
    self.assertEqual(1, 2)
AssertionError: 1 != 2

----------------------------------------------------------------------
Ran 2 tests in 0.000s

[31mFAILED[0m ([31mfailures[0m=[31m1[0m)"""
        ret = pyrg.parse_unittest_result(input_strings.splitlines(1))
        self.assertEqual(ret, result_strings)

    def test_errorroute(self):
        input_strings = """.E
======================================================================
ERROR: test_dummy_error (__main__.TestDummy)
----------------------------------------------------------------------
Traceback (most recent call last):
  File "test/test_pyrg_ng.py", line 143, in test_dummy_error
    self.assertEqual(1, a)
NameError: global name 'a' is not defined

----------------------------------------------------------------------
Ran 2 tests in 0.000s

FAILED (errors=1)
"""
        result_strings = """[32m.[0m[1;33mE[0m
======================================================================
[1;33mERROR[0m: [36mtest_dummy_error (__main__.TestDummy)[0m
----------------------------------------------------------------------
Traceback (most recent call last):
  File "test/test_pyrg_ng.py", line 143, in test_dummy_error
    self.assertEqual(1, a)
NameError: global name 'a' is not defined

----------------------------------------------------------------------
Ran 2 tests in 0.000s

[31mFAILED[0m ([1;33merrors[0m=[1;33m1[0m)"""
        ret = pyrg.parse_unittest_result(input_strings.splitlines(1))
        self.assertEqual(ret, result_strings)

    def test_errorfailroute(self):
        input_strings = """.EF
======================================================================
ERROR: test_dummy_error (__main__.TestDummy)
----------------------------------------------------------------------
Traceback (most recent call last):
  File "test/test_pyrg_ng.py", line 143, in test_dummy_error
    self.assertEqual(1, a)
NameError: global name 'a' is not defined

======================================================================
FAIL: test_dummy_fail (__main__.TestDummy)
----------------------------------------------------------------------
Traceback (most recent call last):
  File "test/test_pyrg_ng.py", line 140, in test_dummy_fail
    self.assertEqual(1, 2)
AssertionError: 1 != 2

----------------------------------------------------------------------
Ran 3 tests in 0.000s

FAILED (failures=1, errors=1)
"""
        result_strings = """[32m.[0m[1;33mE[0m[31mF[0m
======================================================================
[1;33mERROR[0m: [36mtest_dummy_error (__main__.TestDummy)[0m
----------------------------------------------------------------------
Traceback (most recent call last):
  File "test/test_pyrg_ng.py", line 143, in test_dummy_error
    self.assertEqual(1, a)
NameError: global name 'a' is not defined

======================================================================
[31mFAIL[0m: [36mtest_dummy_fail (__main__.TestDummy)[0m
----------------------------------------------------------------------
Traceback (most recent call last):
  File "test/test_pyrg_ng.py", line 140, in test_dummy_fail
    self.assertEqual(1, 2)
AssertionError: 1 != 2

----------------------------------------------------------------------
Ran 3 tests in 0.000s

[31mFAILED[0m ([31mfailures[0m=[31m1[0m, """\
"""[1;33merrors[0m=[1;33m1[0m)"""
        ret = pyrg.parse_unittest_result(input_strings.splitlines(1))
        self.assertEqual(ret, result_strings)


class TestColor(unittest.TestCase):

    def setUp(self):
        self.test_color_define = ['black', 'gray', 'red', 'pink', 'darkred',
                             'green', 'yellowgreen', 'darkgreen', 'brown',
                             'yellow', 'gold', 'blue', 'lightblue', 'darkblue',
                             'magenta', 'lightmagenta', 'darkmagenta',
                             'cyan', 'lightcyan', 'darkcyan', 'silver',
                             'white', 'darksilver']
        None

    def test_colormap_key_nonkey(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(False, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_black(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_gray(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_red(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_pink(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darkred(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_green(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_yellowgreen(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darkgreen(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_brown(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_yellow(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_gold(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_blue(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_lightblue(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darkblue(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_magenta(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_lightmagenta(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darkmagenta(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_cyan(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_lightcyan(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darkcyan(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_silver(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_white(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)

    def test_colormap_key_darksilver(self):
        colorname = self.id().split('_')[-1]
        self.assertEqual(True, colorname in pyrg.COLOR_MAP)


class TestConfig(unittest.TestCase):

    def test_notexist_file(self):
        color_set = pyrg.set_configuration("/home/hogehoge/.pyrgrc")
        self.assertEqual(pyrg.PRINT_COLOR_SET_DEFAULT, color_set)

    def test_config(self):
        config_example = """
[color]
ok = yellowgreen
error = red
fail = blue
function = pink
"""
        temp = NamedTemporaryFile()
        temp.file.write(config_example)
        temp.file.flush()
        color_set = pyrg.set_configuration(temp.name)
        self.assertEqual('yellowgreen', color_set['ok'])
        self.assertEqual('red', color_set['error'])
        self.assertEqual('blue', color_set['fail'])
        self.assertEqual('pink', color_set['function'])
        temp.close()

    def test_config_empty(self):
        config_example = """
[color]
ok =
error =
fail =
function =
"""
        temp = NamedTemporaryFile()
        temp.file.write(config_example)
        temp.file.flush()
        color_set = pyrg.set_configuration(temp.name)
        self.assertEqual(pyrg.PRINT_COLOR_SET_DEFAULT, color_set)
        temp.close()

    def test_config_nonkey_exist(self):
        config_example = """
[color]
ok =
error =
hoge =
fail =
function =
"""
        temp = NamedTemporaryFile()
        temp.file.write(config_example)
        temp.file.flush()
        color_set = pyrg.set_configuration(temp.name)
        self.assertEqual(pyrg.PRINT_COLOR_SET_DEFAULT, color_set)
        temp.close()


if __name__ == '__main__':
    unittest.main()
